#include "SimHost.h"
#include <osg/Notify>

#include <OpenThreads/Thread>
#include <OpenThreads/Barrier>

// System includes
#include <cassert>

#ifdef _WIN32
#include <process.h>
#define getpid() _getpid()
#else
#include <unistd.h>
#endif 

// App includes


SimHost::SimHost()
{
	OSG_ALWAYS<<"SimHost instantiated."<<std::endl;
}

SimHost::~SimHost()
{
	//_channelWorker.clear();
}

void SimHost::initialize()
{
	int numThreads = 6+1;	// 6 channels and one main thread
	OpenThreads::Barrier syncBarrier;

	// Create IG Connectors
	OSG_ALWAYS<<"Creating IG Connectors..."<<std::endl;
	for(int i=0; i<numThreads-1; ++i)
	{
		osg::ref_ptr<ThreadedWorker> threadObject = new ThreadedWorker();
		osg::ref_ptr<IGConnector> IGCon = new IGConnector();
		threadObject->setThreadWorker(IGCon);
		_threadObjects.push_back(threadObject);
		_IGConnectors.push_back(IGCon);
	}
	OSG_ALWAYS<<"...done."<<std::endl;

	// Setup Threading
	OSG_ALWAYS << "Root Thread ID: " << getpid() << std::endl;

	OSG_ALWAYS<<OpenThreads::GetNumberOfProcessors()<<" CPUs found, using "<<(numThreads)<<" of them concurrently."<<std::endl;
	OpenThreads::Thread::SetConcurrency( numThreads );	// Only supported by pthread implementation, otherwise ignored

	OpenThreads::Thread::Init();
	
}

void SimHost::run()
{
	// Launch threads
    for(int i=0;i<static_cast<int>(_threadObjects.size());++i)
	{
		int status;
		
		//thread->setStackSize(1024*256);
		status = _threadObjects[i]->start();
		assert(status == 0);
	}

	// Do something in the main thread
		for(int i=0;i<10000000;i++)	OSG_ALWAYS<<"";
	
	// Signal threads to finish
    for(int i=0;i<static_cast<int>(_threadObjects.size());++i)
	{
		//_threadObjects[i]->setDone(true);	// non blocking
		_threadObjects[i]->cancel();	// Blocking
	}

	for(int i=0;i<100000000;i++)
		OSG_ALWAYS<<"";

}